@@ -3,6 +3,7 @@ use strict;
 use warnings;
 use Hash::MultiValue;
 use Protocol::HTTP2::Constants qw(:states :endpoints);
+use Protocol::HTTP2::HeaderCompression qw( headers_decode );
 use Protocol::HTTP2::Trace qw(tracer);
 
 # Streams related part of Protocol::HTTP2::Conntection
@@ -29,7 +30,7 @@ sub new_peer_stream {
     $self->{last_peer_stream} = $stream_id;
     $self->{streams}->{$stream_id} = { 'state' => IDLE };
     $self->{on_new_peer_stream}->($stream_id)
-      if exists $self->{on_new_peer_stream} && $self->{type} == SERVER;
+      if exists $self->{on_new_peer_stream};
 
     return $self->{last_peer_stream};
 }
@@ -41,6 +42,8 @@ sub stream {
     $self->{streams}->{$stream_id};
 }
 
+# stream_state ( $self, $stream_id, $new_state?, $pending? )
+
 sub stream_state {
     my $self      = shift;
     my $stream_id = shift;
@@ -48,26 +51,49 @@ sub stream_state {
     my $s = $self->{streams}->{$stream_id};
 
     if (@_) {
-        my $new_state = shift;
+        my ( $new_state, $pending ) = @_;
 
-        $self->{on_change_state}->( $stream_id, $s->{state}, $new_state )
-          if exists $self->{on_change_state};
+        if ($pending) {
+            $self->stream_pending_state( $stream_id, $new_state );
+        }
+        else {
+            $self->{on_change_state}->( $stream_id, $s->{state}, $new_state )
+              if exists $self->{on_change_state};
 
-        $s->{state} = $new_state;
+            $s->{state} = $new_state;
 
-        # Exec callbacks for new state
-        $s->{cb}->{ $s->{state} }->()
-          if exists $s->{cb} && exists $s->{cb}->{ $s->{state} };
+            # Exec callbacks for new state
+            $s->{cb}->{ $s->{state} }->()
+              if exists $s->{cb} && exists $s->{cb}->{ $s->{state} };
 
-        # Cleanup
-        if ( $new_state == CLOSED ) {
-            $s = $self->{streams}->{$stream_id} = { state => CLOSED };
+            # Cleanup
+            if ( $new_state == CLOSED ) {
+                $s = $self->{streams}->{$stream_id} = { state => CLOSED };
+            }
         }
     }
 
     $s->{state};
 }
 
+sub stream_pending_state {
+    my $self      = shift;
+    my $stream_id = shift;
+    return undef unless exists $self->{streams}->{$stream_id};
+    my $s = $self->{streams}->{$stream_id};
+    $s->{pending_state} = shift if @_;
+    $s->{pending_state};
+}
+
+sub stream_promised_sid {
+    my $self      = shift;
+    my $stream_id = shift;
+    return undef unless exists $self->{streams}->{$stream_id};
+    my $s = $self->{streams}->{$stream_id};
+    $s->{promised_sid} = shift if @_;
+    $s->{promised_sid};
+}
+
 sub stream_cb {
     my ( $self, $stream_id, $state, $cb ) = @_;
 
@@ -87,6 +113,18 @@ sub stream_data {
     $s->{data};
 }
 
+# Header Block -- The entire set of encoded header field representations
+sub stream_header_block {
+    my $self      = shift;
+    my $stream_id = shift;
+    return undef unless exists $self->{streams}->{$stream_id};
+    my $s = $self->{streams}->{$stream_id};
+
+    $s->{header_block} .= shift if @_;
+
+    $s->{header_block};
+}
+
 sub stream_headers {
     my $self      = shift;
     my $stream_id = shift;
@@ -94,13 +132,30 @@ sub stream_headers {
     $self->{streams}->{$stream_id}->{headers};
 }
 
+sub stream_pp_headers {
+    my $self      = shift;
+    my $stream_id = shift;
+    return undef unless exists $self->{streams}->{$stream_id};
+    $self->{streams}->{$stream_id}->{pp_headers};
+}
+
 sub stream_headers_done {
     my $self      = shift;
     my $stream_id = shift;
     return undef unless exists $self->{streams}->{$stream_id};
     my $s = $self->{streams}->{$stream_id};
+
+    my $res =
+      headers_decode( $self, \$s->{header_block}, 0,
+        length $s->{header_block} );
+
     tracer->debug("Headers done for stream $stream_id\n");
 
+    return undef unless defined $res;
+
+    # Clear header_block
+    $s->{header_block} = '';
+
     my $rs = $self->decode_context->{reference_set};
     my $eh = $self->decode_context->{emitted_headers};
 
@@ -110,10 +165,19 @@ sub stream_headers_done {
         next if grep { $_ eq $value } $h->get_all($key);
         $h->add( $key, $value );
     }
-    $s->{headers} = [ $h->flatten ];
 
-    # Clear emitted headers;
+    if ( $s->{promised_sid} ) {
+        $self->{streams}->{ $s->{promised_sid} }->{pp_headers} =
+          [ $h->flatten ];
+    }
+    else {
+        $s->{headers} = [ $h->flatten ];
+    }
+
+    # Clear emitted headers
     $self->decode_context->{emitted_headers} = [];
+
+    return 1;
 }
 
 1;
